# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2010-2013 Elico Corp. All Rights Reserved.
#    Augustin Cisterne-Kaas <augustin.cisterne-kaaas@elico-corp.com>
#    Eric Caudal <eric.caudal@elico-corp.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import orm, fields


class res_intercompany(orm.Model):
    _name = "res.intercompany"
    _description = "Inter-Company"

    def get_intercompany(self, cr, uid, obj_id,
                         obj_class_name, obj_name, context=None):
        """
        get company from and to
        """

        assert isinstance(obj_id, int) or isinstance(obj_id, long)
        obj = self.pool.get(obj_class_name).browse(
            cr,
            uid,
            obj_id,
            context=None)
        company = obj.company_id
        ic_uid_ids = None
        company_to_ids = []
        for ic in company.intercompany_ids:
            if ic.obj2obj == obj_name:
                company_to_ids.append(ic.company_to.id)
                ic_uid_ids = ic.intercompany_uid.id
        if company.intercompany_ids:
            ic_uid_ids = company.intercompany_ids[0].intercompany_uid.id
        return obj.company_id.id, company_to_ids, ic_uid_ids

    def _check_intercompany_user(self, cr, uid, ids, context=None):
        for ic in self.browse(cr, uid, ids, context=context):
            if not ic.intercompany_uid:
                return False
        return True

    _columns = {
        'company_from': fields.many2one('res.company', 'Company From',
                                        required=True, ondelete='cascade'),
        'company_to': fields.many2one('res.company', 'Company', required=True,
                                      ondelete='cascade'),
        'obj2obj': fields.selection(
            [('so2po', 'SO to PO'),
             ('po2so', 'PO to SO'),
             ('is2do', 'IS to DO'),
             ('do2is', 'DO to IS'),
             ('ci2si', 'CI to SI'),
             ('si2ci', 'SI to CI')],
            string="Relation", required=True),
        'status': fields.selection(
            [('draft', 'On Creation'),
             ('confirm', 'On Confirmation'),
             ('cancel', 'On Cancel'),
             ('unlink', 'On Deletion')],
            string='Status', required=True),
        'direction': fields.selection(
            [('bidirectional', 'Bidirectional'),
             ('regular', 'Regular'),
             ('inverse', 'Inverse')],
            'Modification Direction', required=True),
        'auto_confirm': fields.boolean('Automatic Confirm'),
        'intercompany_uid': fields.related(
            'company_to', 'intercompany_uid', type='many2one',
            relation='res.users', readonly=True, string='IC User'),
    }
    _defaults = {
        'auto_confirm': lambda *a: True,
        'direction': lambda *a: 'bidirectional',
    }

    _constraints = [
        (_check_intercompany_user, 'Please set IC user for the Company first',
            ['intercompany_uid'])
    ]

    def check_need_create_intercompany_object(
            self, cr, uid, company_from, company_to, o2o_field_name,
            node=None, return_list_type=False):
        """ @company_from
            @company_to
            @o2o_field_name  file name.Example, so2po po2so ,,,,
            @node  value of  o2o_field_name ,  draft, confirm
            @return_list_type,  the return type is list or boolean
        """

        if type(company_to) != list:
            company_to = [company_to]

        intercompany_ids = self.search(
            cr, uid,
            [('company_from', '=', company_from),
             ('company_to', 'in', company_to),
             ('obj2obj', '=', o2o_field_name),
             ('status', '=', node)])
        if return_list_type:
            return intercompany_ids
        else:
            return intercompany_ids and True or False

    def check_need_automatic_confirm(
            self, cr, uid, company_from, company_to,
            o2o_field_name, node=None, return_list_type=False):
        """
        check is need to automatic confirm internal company object
        """
        intercompany_ids = self.search(
            cr, uid,
            [('company_from', '=', company_from),
             ('company_to', '=', company_to),
             (o2o_field_name, '=', node),
             ('auto_confirm', '=', True)])
        return intercompany_ids and True or False


class res_company(orm.Model):
    _inherit = "res.company"

    _columns = {
        'intercompany_ids': fields.one2many(
            'res.intercompany', 'company_from', 'Inter-Company Setup'),
        'intercompany_uid': fields.many2one(
            'res.users', 'IC User',
            required=True,
            domain="[('company_id', '=', id)]",
            help="User to create update unlink IC records"),
        'intercompany_shop_id': fields.many2one(
            'sale.shop', 'IC Default location',
            domain="[('company_id', '=', id)]")
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
