# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2010-2013 Elico Corp. All Rights Reserved.
#     Jon Chow <jon.chow@elico-corp.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import osv, fields
from openerp import SUPERUSER_ID


class account_invoice(osv.osv):
    _inherit = "account.invoice"
    _name = "account.invoice"

    _columns = {
        'is_locked': fields.boolean('Locked for Intercompany'),
        'ic_create': fields.boolean('Intercompany Invoice generated'),
        'invoice_ic_id': fields.many2one('account.invoice', 'Invoice'),
    }

    def get_from_to_company(self, cr, uid, ids, context=None):
        """ get company from and to """
        assert len(ids) == 1
        cr.execute(""" 
            SELECT
                inv.company_id AS from_c, cmp.id AS to_c
            FROM account_invoice AS inv
                LEFT JOIN res_partner as part ON (inv.partner_id = part.id)
                LEFT JOIN res_company as cmp  ON (part.id = cmp.partner_id )
            WHERE inv.id=%s """, (ids[0],),
        )
        return cr.fetchone()

    def need_to_create_intercompany_invoice(self, cr, uid, ids, o2o_field_name=None, node=None, context=None):
        """
        @o2o_field_name  
        @node,  draft' or 'confirm', res.intercompany.so2po value ,
        return boolean
        """
        company_from, company_to = self.get_from_to_company(cr, uid, ids, context=context)
        need = self.pool.get('res.intercompany').check_need_create_intercompany_object(cr, uid, company_from, company_to, o2o_field_name, node)
        return need

    def action_draft(self, cr, uid, ids, context=None):
        """
        Picking wkf Draft action,
        Use Administrator operate intercompany action
        """
        assert len(ids) == 1
        self.action_intercompany(cr, SUPERUSER_ID, ids, node='draft', context=context)
        return True

    def action_intercompany(self, cr, uid, ids, node=None, context=None):
        """
        if CI, create SI
        if SI, create CI
        @node, draft or confirm,  
        """
        assert len(ids) == 1
        cr.execute(""" SELECT type FROM account_invoice WHERE id=%s """,(ids[0],))
        invoice_type = cr.fetchone()[0]

        if invoice_type in ['in_invoice', 'out_invoice']:
            #if type=in, check is2do,  type=out,
            o2o_field_name = invoice_type=='in_invoice' and 'ci2si' or 'si2ci'
            if self.need_to_create_intercompany_invoice(cr, uid, ids, o2o_field_name, node=node, context=context):
                self.create_intercompany_invoice(cr, uid, ids, self_type=invoice_type, context=None)

    def create_intercompany_invoice(self, cr, uid, ids, self_type=None, context=None):
        invoice    = self.browse(cr, uid, ids[0])
        company_from, company_to = self.get_from_to_company(cr, uid, ids, context=context)
        data       = self.prepare_intercompany_invoice(cr, uid, invoice, company_to, context=context)
        invoice_id = self.create(cr, uid, data, context=context)
        return invoice_id

    def prepare_intercompany_invoice(self, cr, uid, invoice, company_to,context=None):
        """ prepare  intercompany invoice data
            @invoice, browse of account.invoice 
            @company_to,  intercompany_invoice company_id
            return {}, data of create invoice
        """
        line_pool    = self.pool.get('account.invoice.line')
        journal_pool = self.pool.get('account.journal')
        period_pool  = self.pool.get('account.period')
        part         = invoice.company_id.partner_id

        #Invoice Type and acc_id
        hash = {
            'in_invoice': {'create_type':'out_invoice', 'value_reference_name':'property_account_payable'  },
            'out_invoice':{'create_type':'in_invoice' , 'value_reference_name':'property_account_receivable' }
        }
        acc_id = part.property_account_receivable.id
        cr.execute("SELECT value_reference FROM ir_property WHERE name=%s AND res_id='res.partner,%s' AND company_id=%s",(hash[invoice.type]['value_reference_name'],part.id,company_to, ))
        prop = cr.fetchone()
        if prop and prop[0]:
            acc_id = prop[0].split('account.account,')[1]

        #Invoice Journal
        type2journal = {'out_invoice': 'sale', 'in_invoice': 'purchase', 'out_refund': 'sale_refund', 'in_refund': 'purchase_refund'}        
        res          = journal_pool.search(cr, uid, [('type', '=', type2journal.get(hash[invoice.type]['create_type'], 'sale')), ('company_id', '=', company_to)], limit=1)
        journal_id   = res and res[0] or False
        # We assume periods in Inter-companies have the same code. 
        period_ids = period_pool.search(cr, uid, [('code', '=', invoice.period_id.code), ('company_id', '=', company_to)], limit=1)
        period_id  = period_ids and period_ids[0] or False

        lines_data=[(5,)]
        for invoice_line in invoice.invoice_line:
            data_line = {
                # invoice_id:  ,
                'name':       invoice_line.name,
                'origin':     invoice_line.origin,
                'sequence':   invoice_line.sequence,
                'uos_id':     invoice_line.uos_id     and invoice_line.uos_id.id     or False,
                'product_id': invoice_line.product_id and invoice_line.product_id.id or False,
                'account_id': acc_id,
                'price_unit': invoice_line.price_unit,
                'quantity':   invoice_line.quantity,
                'discount':   invoice_line.discount,
                'company_id': company_to,
            }
            lines_data.append( (0, 0, data_line) )

        invoice_data = {
            'name':           invoice.name,
            'origin':         invoice.number and 'IC:' + str(invoice.number) or '',
            'supplier_invoice_number': invoice.supplier_invoice_number,
            'type':           hash[invoice.type]['create_type'],
            'reference':      invoice.reference,
            'reference_type': invoice.reference_type,
            'comment':        invoice.comment,
            'date_invoice':   invoice.date_invoice,
            'date_due':       invoice.date_due,
            'partner_id':     part.id,
            'payment_term':   invoice.payment_term and invoice.payment_term.id or False,
            'period_id':      period_id,
            'account_id':     acc_id,
            'journal_id':     journal_id,
            'company_id':     company_to , 
            'invoice_ic_id':  invoice.id,
            'is_locked':      True,
            'ic_create':      True,       
            'invoice_line':   lines_data,      
             #'currency_id':   invoice.currency_id and invoice.currency_id.id or False,
        }
        return invoice_data

    def write(self, cr, uid, ids, values, context=None):
        res = super(account_invoice, self).write(cr, uid, ids, values, context=context)
        if type(ids) != list:
            ids = [ids]
        self.update_intercompany_invoice(cr, uid, ids, context=None)
        return res

    def unlink(self, cr, uid, ids, context=None):
        res = super(account_invoice, self).unlink(cr, uid, ids, context=context)
        if type(ids) != type([]):
            ids = [ids]

        for inv_id in ids:
            inv_ic_ids = self.search(cr, uid, [('invoice_ic_id','=',inv_id)], context=context)
            if inv_ic_ids:
                self.unlink(cr, uid, inv_ic_ids, context=context)
        return res

    def update_intercompany_invoice(self, cr, uid, ids, context=None):
        for invoice in self.browse(cr, uid, ids, context):
            invoice_ic_id = self.search(cr, uid, [('invoice_ic_id','=',invoice.id )])
            if invoice_ic_id:
                company_from, company_to = self.get_from_to_company(cr, uid, [invoice.id,] ,context=context)
                data = self.prepare_intercompany_invoice(cr, uid, invoice, company_to, context=context)
                self.write(cr, uid, invoice_ic_id, data)
        return True

    def invoice_validate(self, cr, uid, ids, context=None):
        res = super(account_invoice, self).invoice_validate(cr, uid, ids, context=context)
        if res:
            self.action_intercompany(cr, uid, ids, node='confirm', context=context)
        return res

account_invoice()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: