# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2010-2013 Elico Corp. All Rights Reserved.
#    Augustin Cisterne-Kaas <augustin.cisterne-kaaas@elico-corp.com>
#    Eric Caudal <eric.caudal@elico-corp.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp import SUPERUSER_ID
from openerp.osv import fields, orm, osv
from openerp.tools.translate import _
from openerp import netsvc
from icops.order import icops_order
from openerp import netsvc


class purchase_order(orm.Model):
    _inherit = 'purchase.order'
    _columns = {
        'is_locked': fields.boolean('Locked for Intercompany'),
        'ic_create': fields.boolean('Intercompany PO generated'),
        'sale_id': fields.many2one('sale.order', 'Sales Order'),
    }

    _sql_constraints = [(
        'sale_id_unique', 'unique(sale_id)',
        'Sale Id already exists')]

    def action_draft(self, cr, uid, ids, context=None):
        """
        function of wkf active draft,
        intercompany action operate by Administrator
        SUPERUSER_ID used to search, browse kinds of object.
        but not used of create, write anything
        """

        assert len(ids) == 1
        context = context or {}
        order = self.read(cr, uid, ids[0], ['sale_id'])
        if order['sale_id']:
            context['is_locked'] = True
            context['is_external'] = True
        icops = icops_order(self)
        icops.create_intercompany(cr, uid, ids,
                                  node='draft', context=context)
        return True

    def write(self, cr, uid, ids, values, context=None):
        if not isinstance(ids, list):
            ids = [ids]
        context = context or {}

        res = super(purchase_order, self).write(
            cr, uid, ids, values, context=context)
        if 'is_locked' not in context:
            if 'is_external' in context:
                context['is_locked'] = True
            icops = icops_order(self)
            try:
                icops.action(
                    cr, uid, ids, values, context=context)
            except:
                raise
        return res

    def unlink(self, cr, uid, ids, context=None):
        if not isinstance(ids, list):
            ids = [ids]
        context = context or {}
        if 'is_locked' not in context:
            if 'is_external' in context:
                context['is_locked'] = True
            icops = icops_order(self)
            try:
                icops.action(cr, uid, ids, state="unlink",
                             context=context)
            except:
                raise

        return super(purchase_order, self).unlink(
            cr, uid, ids, context=context)

    def action_cancel(self, cr, uid, ids, context=None):
        wf_service = netsvc.LocalService("workflow")
        for purchase in self.browse(cr, uid, ids, context=context):
            for pick in purchase.picking_ids:
                if pick.state not in ('draft', 'cancel'):
                    raise osv.except_osv(
                        _('Unable to cancel this purchase order.'),
                        _('First cancel all receptions related\
                           to this purchase order.'))
            for pick in purchase.picking_ids:
                wf_service.trg_validate(
                    uid, 'stock.picking', pick.id, 'button_cancel', cr)
            for inv in purchase.invoice_ids:
                if inv and inv.state not in ('cancel', 'draft'):
                    raise osv.except_osv(
                        _('Unable to cancel this purchase order.'),
                        _('You must first cancel all receptions related to \
                            this purchase order.'))
                if inv:
                    wf_service.trg_validate(
                        uid, 'account.invoice', inv.id, 'invoice_cancel', cr)
        self.write(cr, uid, ids, {'state': 'cancel'}, context=context)

        for (id, name) in self.name_get(cr, uid, ids):
            wf_service.trg_validate(
                uid, 'purchase.order', id, 'purchase_cancel', cr)
        return True

    def _prepare_IC(self, cr, uid, id,
                    company_to, ic_uid, context=None):
        """
        according the PO and company_to, prepare the SO data,
        It can be used to create or update SO record.
        @po  SO browse record
        @company_to      company_id
        @ic_uid
        return { }, value of SO data
        """
        if isinstance(company_to, list):
            company_to = company_to[0]
        order = self.read(cr, uid, id, ['name', 'company_id', 'order_line',
                                        'state'])
        company_pool = self.pool.get('res.company')
        company = company_pool.read(
            cr, uid, order['company_id'][0], ['partner_id'])
        partner_pool = self.pool.get('res.partner')
        customer = partner_pool.read(
            cr, ic_uid, company['partner_id'][0],
            ['property_product_pricelist',
             'property_account_position',
             'property_payment_term',
             'user_id'])

        addr = partner_pool.address_get(cr, uid, [customer['id']],
                                        ['delivery', 'invoice', 'contact'])

        pricelist = (customer['property_product_pricelist'][0]
                     if customer['property_product_pricelist']
                     else False)

        fiscal_position = (customer['property_account_position'][0]
                           if customer['property_account_position']
                           else False)
        payment_term = (customer['property_payment_term'][0]
                        if customer['property_payment_term']
                        else False)

        salesman = customer['user_id'][0] if customer['user_id'] else ic_uid
        company = company_pool.read(
            cr, ic_uid, company_to, ['intercompany_shop_id'])

        if not 'intercompany_shop_id' in company:
            raise osv.except_osv(
                _('Error!'),
                _('There is no default shop for the destination company!'))

        shop_id = company['intercompany_shop_id'][0]
        sol_datas = [(5,)]
        pol_pool = self.pool.get('purchase.order.line')
        i = 0
        for line in pol_pool.read(
                cr, uid, order['order_line'],
                ['name', 'ic_pol_id', 'product_qty',
                 'product_id', 'product_uom', 'price_unit',
                 'state']):
            line_data = (0, 0, {
                #'order_id':       so_id,
                'name': line['name'],
                'ic_pol_id': line['id'],
                'product_uom_qty': line['product_qty'],
                'product_id': line['product_id'][0],
                'product_uom': line['product_uom'][0],
                'price_unit': line['price_unit']
            })
            sol_datas.append(line_data)
        state = "draft" if order['state'] != "approved" else "progress"
        so_data = {
            'partner_id': customer['id'],
            'pricelist_id': pricelist,
            'partner_invoice_id': addr['invoice'],
            # 'partner_order_id': addr['contact'],
            'partner_shipping_id': addr['delivery'],
            'payment_term': payment_term,
            'fiscal_position': fiscal_position,
            'user_id': salesman,
            'purchase_id': order['id'],
            'ic_create': True,
            'shop_id': shop_id,
            'origin': 'IC:' + str(order['name']),
            'order_line': sol_datas,
            'state': state
        }
        if 'is_external' in context:
            so_data = {
                'order_line': sol_datas,
                'state': state
            }

        return so_data

    def copy(self, cr, uid, id, default=None, context=None):
        if not default:
            default = {}
        default.update({
            'sale_id': None
        })
        return super(purchase_order, self).copy(cr, uid, id, default, context)

    def wkf_confirm_order(self, cr, uid, ids, context=None):
        if not isinstance(ids, list):
            ids = [ids]
        context = context or {}
        if 'is_external' not in context:
            icops = icops_order(self)
            try:
                icops.action(
                    cr, uid, ids, state='confirm', context=context)
            except:
                raise
            todo = []
            sol_pool = self.pool.get('sale.order.line')
            for po in self.read(cr, uid, ids, ['order_line']):
                if not po['order_line']:
                    raise osv.except_osv(_('Error!'), _(
                        'You cannot confirm a purchase order without any\
                         purchase order line.'))
                for line in sol_pool.read(cr, uid, po['order_line'], ['state']):
                    if line['state'] == 'draft':
                        todo.append(line.id)

            self.pool.get('purchase.order.line').action_confirm(
                cr, uid, todo, context)
            for id in ids:
                self.write(
                    cr, uid, [id], {'state': 'confirmed', 'validator': uid},
                    context=context)
        return True


class purchase_order_line(orm.Model):
    _inherit = 'purchase.order.line'
    _columns = {
        'ic_sol_id': fields.many2one('sale.order.line', 'IC SOL'),
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
