# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2010-2013 Elico Corp. All Rights Reserved.
#    Augustin Cisterne-Kaas <augustin.cisterne-kaaas@elico-corp.com>
#    Eric Caudal <eric.caudal@elico-corp.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields, orm, osv
from openerp.tools.translate import _
from icops.order import icops_order
from openerp import SUPERUSER_ID
from openerp import netsvc
from collections import namedtuple


class sale_order(orm.Model):
    _inherit = 'sale.order'
    _columns = {
        'is_locked': fields.boolean('Locked for Intercompany'),
        'ic_create': fields.boolean('Intercompany SO generated'),
        'purchase_id': fields.many2one('purchase.order', 'Purchase Order'),
    }

    _sql_constraints = [(
        'purchase_id_unique', 'unique(purchase_id)',
        'Purchase Id already exists')]

    def action_draft(self, cr, uid, ids, context=None):
        """
        SO wkf Draft action,
        Use Administrator operate intercompany action
        """
        assert len(ids) == 1
        if context is None:
            context = {}
        order = self.browse(cr, SUPERUSER_ID, ids[0], context=context)
        if order.purchase_id:
            if order.purchase_id:
                context['is_locked'] = True
                context['is_external'] = True
        icops = icops_order(self)
        icops.create_intercompany(cr, uid, ids, context=context)
        return True

    def write(self, cr, uid, ids, values, context=None):
        if not isinstance(ids, list):
            ids = [ids]
        context = context or {}
        res = super(sale_order, self).write(
            cr, uid, ids, values, context=context)
        if 'is_locked' not in context:
            if 'is_external' in context:
                context['is_locked'] = True
            icops = icops_order(self)
            try:
                icops.action(cr, uid, ids, values,
                             context=context)
            except:
                raise
        return res

    def unlink(self, cr, uid, ids, context=None):
        if not isinstance(ids, list):
            ids = [ids]
        if not isinstance(context, dict):
            context = {}
        if 'is_locked' not in context:
            if 'is_external' in context:
                context['is_locked'] = True
            icops = icops_order(self)
            try:
                icops.action(cr, uid, ids, state="unlink",
                             context=context)
            except:
                raise
        return super(sale_order, self).unlink(
            cr, uid, ids, context=context)

    def action_cancel(self, cr, uid, ids, context=None):
        wf_service = netsvc.LocalService("workflow")
        context = context or {}
        sale_order_line_obj = self.pool.get('sale.order.line')
        for sale in self.browse(cr, uid, ids, context=context):
            for inv in sale.invoice_ids:
                if inv.state not in ('draft', 'cancel'):
                    raise osv.except_osv(
                        _('Cannot cancel this sales order!'),
                        _('First cancel all invoices attached\
                            to this sales order.'))
            for r in self.read(cr, uid, ids, ['invoice_ids']):
                for inv in r['invoice_ids']:
                    wf_service.trg_validate(
                        uid, 'account.invoice', inv, 'invoice_cancel', cr)
                    sale_order_line_obj.write(
                        cr, uid, [l.id for l in sale.order_line],
                        {'state': 'cancel'})
            self.write(cr, uid, ids, {'state': 'cancel'}, context=context)
        return True

    def action_wait(self, cr, uid, ids, context=None):
        if not isinstance(ids, list):
            ids = [ids]
        context = context or {}
        if 'is_external' not in context:
            icops = icops_order(self)
            try:
                icops.action(
                    cr, uid, ids, state='confirm', context=context)
            except:
                raise
            return super(sale_order, self).action_wait(
                cr, uid, ids, context=context)

    def _prepare_IC(self, cr, uid, id,
                    company_to, ic_uid, context=None):

        """
        prepare the intercompany PO data,
        it can be used to create or update PO record
        @ so  SO browse record
        @ company_to  company_id
        @ic_uid  Internal company user id
        """
        if isinstance(company_to, list):
            company_to = company_to[0]
        # to avoid many SQL requests
        OrderStruct = namedtuple('OrderStruct', 'company_id')
        CompanyStruct = namedtuple('CompanyStruct', 'security_lead')
        LineStruct = namedtuple('LineStruct', 'delay')

        order = self.read(cr, uid, id,
                          ['order_line', 'date_order', 'company_id',
                           'state', 'name'])
        company_pool = self.pool.get('res.company')
        company = company_pool.read(
            cr, uid, order['company_id'][0], ['partner_id', 'security_lead'])

        fake_company_obj = CompanyStruct(
            security_lead=company['security_lead'] or 0)
        fake_so_obj = OrderStruct(company_id=fake_company_obj)
        supplier = self.pool.get('res.partner').read(
            cr, ic_uid, company['partner_id'][0],
            ['property_product_pricelist_purchase',
             'property_account_position',
             'property_supplier_payment_term'])
        pricelist = (supplier['property_product_pricelist_purchase'][0]
                     if supplier['property_product_pricelist_purchase']
                     else False)

        fiscal_position = (supplier['property_account_position'][0]
                           if supplier['property_account_position']
                           else False)
        payment_term = (supplier['property_supplier_payment_term'][0]
                        if supplier['property_supplier_payment_term']
                        else False)

        company = company_pool.read(
            cr, ic_uid, company_to, ['intercompany_shop_id'])
        intercompany_shop = self.pool.get('sale.shop').read(
            cr, ic_uid, company['intercompany_shop_id'][0], ['warehouse_id'])
        warehouse = self.pool.get('stock.warehouse').read(
            cr, ic_uid, intercompany_shop['warehouse_id'][0], ['lot_stock_id'])

        if not 'lot_stock_id' in warehouse:
            raise osv.except_osv(
                _('Error!'),
                _('prepare_intercompany_po_data,not found location_id'))

        location_id = warehouse['lot_stock_id'][0]
        polines_data = [(5,)]
        sol_pool = self.pool.get('sale.order.line')
        for line in sol_pool.read(
                cr, uid, order['order_line'],
                ['name', 'product_uom_qty', 'product_id', 'product_uom',
                 'price_unit', 'delay']):

            fake_line_obj = LineStruct(delay=line['delay'])
            date_planned = self._get_date_planned(
                cr, uid, fake_so_obj,
                fake_line_obj,
                order['date_order'], context=context)
            pol_data = (0, 0, {
                'ic_sol_id': line['id'],
                'name': line['name'],
                'product_qty': line['product_uom_qty'],
                'product_id': line['product_id'][0],
                'product_uom': line['product_uom'][0],
                'price_unit': line['price_unit'],
                'date_planned': date_planned,
            })
            polines_data.append(pol_data)
        company_id = company_to
        state = "draft" if order['state'] != "progress" else "approved"
        po_data = {
            'partner_id': supplier['id'],
            'pricelist_id': pricelist,
            'payment_term_id': payment_term,
            'fiscal_position': fiscal_position,
            'sale_id': order['id'],
            'ic_create': True,
            'company_id': company['id'],
            'location_id': location_id,
            'origin': 'IC:' + str(order['name']),
            'order_line': polines_data,
            'state': state
        }
        if 'is_external' in context:
            po_data = {
                'order_line': polines_data,
                'state': state
            }

        return po_data

    def copy(self, cr, uid, id, default=None, context=None):
        if not default:
            default = {}
        default.update({
            'purchase_id': None
        })
        return super(sale_order, self).copy(cr, uid, id, default, context)


class sale_order_line(orm.Model):
    _inherit = 'sale.order.line'
    _columns = {
        'ic_pol_id': fields.many2one('purchase.order.line',
                                     'Intercompany PO Line'),
    }
#vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
