import mock
import unittest2
from datetime import datetime, timedelta

import openerp
import openerp.tests.common as common
from openerp.osv import osv

DB = common.DB
ADMIN_USER_ID = common.ADMIN_USER_ID


class test_icops_sale(common.TransactionCase):

    def setUp(self):
        super(test_icops_sale, self).setUp()
        # get super user
        self.m = self.registry('ir.model.data')
        self.user_pool = self.registry('res.users')
        self.super_user = self.user_pool.browse(self.cr, self.uid, self.uid)
        # creates companies
        self.company_1 = self.m.get_object(
            self.cr, self.uid, 'base', 'main_company')

        self.company_2 = self.m.get_object(
            self.cr, self.uid, 'multi_company', 'res_company_oerp_be')

        self.user_1 = self.m.get_object(
            self.cr, self.uid, 'base_intercompany', 'user_icops1')

        self.user_2 = self.m.get_object(
            self.cr, self.uid, 'base_intercompany', 'user_icops2')

        self.shop_1 = self.m.get_object(
            self.cr, self.uid, 'base_intercompany',
            'sale_shop_icops_1')

        self.shop_2 = self.m.get_object(
            self.cr, self.uid, 'base_intercompany',
            'sale_shop_icops_2')

        company_pool = self.registry('res.company')
        company_pool.write(
            self.cr, self.uid, self.company_1.id,
            {'intercompany_uid': self.user_1.id,
             'intercompany_shop_id': self.shop_1.id})

        company_pool.write(
            self.cr, self.uid, self.company_2.id,
            {'intercompany_uid': self.user_2.id,
             'intercompany_shop_id': self.shop_2.id})

        self.icops_pool = self.registry('res.intercompany')
        self.so_pool = self.registry('sale.order')
        self.po_pool = self.registry('purchase.order')

    def test_01_so_creates_po(self):

        self.icops_pool.create(
            self.cr, self.uid,
            {'company_from': self.company_1.id,
             'company_to': self.company_2.id,
             'obj2obj': 'so2po',
             'status': 'draft',
             'direction': 'regular'})
        sale_id = self.so_pool.create(self.cr, self.user_1.id, {
            'partner_id': self.company_2.partner_id.id,
            'partner_invoice_id': self.company_2.partner_id.id,
            'partner_shipping_id': self.company_2.partner_id.id,
            'shop_id': self.shop_1.id,
            'date_order': datetime.now().strftime('%Y-%m-%d'),
            'pricelist_id': self.user_1.property_product_pricelist.id
        })
        so = self.so_pool.browse(self.cr, self.user_1.id, sale_id)
        self.assertTrue(so.purchase_id.id)
        po = self.po_pool.browse(self.cr, self.uid, so.purchase_id.id)
        self.assertIsNotNone(po.id)
        self.assertEqual(so.purchase_id.id, po.id)
        self.assertEqual(po.sale_id.id, so.id)

    def test_02_creates_normal_so(self):

        self.icops_pool.create(
            self.cr, self.uid,
            {'company_from': self.company_1.id,
             'company_to': self.company_2.id,
             'obj2obj': 'so2po',
             'status': 'draft',
             'direction': 'regular'})
        sale_id = self.so_pool.create(self.cr, self.user_1.id, {
            'partner_id': self.company_1.partner_id.id,
            'partner_invoice_id': self.company_1.partner_id.id,
            'partner_shipping_id': self.company_1.partner_id.id,
            'shop_id': self.shop_1.id,
            'date_order': datetime.now().strftime('%Y-%m-%d'),
            'pricelist_id': self.user_1.property_product_pricelist.id
        })
        so = self.so_pool.browse(self.cr, self.user_1.id, sale_id)

        self.assertFalse(so.purchase_id.id)

    def test_03_so_creates_po_updates_so_line_regular(self):
        product = self.m.get_object(
            self.cr, self.uid, 'product', 'product_product_1')
        self.icops_pool.create(
            self.cr, self.uid,
            {'company_from': self.company_1.id,
             'company_to': self.company_2.id,
             'obj2obj': 'so2po',
             'status': 'draft',
             'direction': 'regular'})
        sale_id = self.so_pool.create(self.cr, self.user_1.id, {
            'partner_id': self.company_2.partner_id.id,
            'partner_invoice_id': self.company_2.partner_id.id,
            'partner_shipping_id': self.company_2.partner_id.id,
            'shop_id': self.shop_1.id,
            'date_order': datetime.now().strftime('%Y-%m-%d'),
            'pricelist_id': self.user_1.property_product_pricelist.id
        })
        so = self.so_pool.browse(self.cr, self.user_1.id, sale_id)

        po = so.purchase_id
        self.assertEqual(so.order_line, po.order_line)
        sol = [(0, False, {
            'product_uos_qty': 1, 'product_id': product.id,
            'product_uom': 1, 'price_unit': 885,
            'product_uom_qty': 1, 'th_weight': 0, 'product_uos': False,
            'tax_id': [[6, False, []]],
            'name': product.name})]
        self.so_pool.write(
            self.cr, self.user_1.id, so.id, {'order_line': sol})
        so = self.so_pool.browse(
            self.cr, self.user_1.id, so.id)

        self.assertEqual(len(so.order_line), 1)

        po = self.po_pool.browse(
            self.cr, self.user_2.id, po.id)
        self.assertEqual(
            so.order_line[0].product_id, po.order_line[0].product_id)
        self.assertEqual(
            so.order_line[0].product_uom_qty, po.order_line[0].product_qty)

    def test_04_so_creates_po_updates_so_line_inverse(self):
        product = self.m.get_object(
            self.cr, self.uid, 'product', 'product_product_1')
        self.icops_pool.create(
            self.cr, self.uid,
            {'company_from': self.company_1.id,
             'company_to': self.company_2.id,
             'obj2obj': 'so2po',
             'status': 'draft',
             'direction': 'inverse'})
        sale_id = self.so_pool.create(self.cr, self.user_1.id, {
            'partner_id': self.company_2.partner_id.id,
            'partner_invoice_id': self.company_2.partner_id.id,
            'partner_shipping_id': self.company_2.partner_id.id,
            'shop_id': self.shop_1.id,
            'date_order': datetime.now().strftime('%Y-%m-%d'),
            'pricelist_id': self.user_1.property_product_pricelist.id
        })
        so = self.so_pool.browse(self.cr, self.user_1.id, sale_id)

        po = so.purchase_id
        self.assertEqual(so.order_line, po.order_line)
        sol = [(0, False, {
            'product_uos_qty': 1, 'product_id': product.id,
            'product_uom': 1, 'price_unit': 885,
            'product_uom_qty': 1, 'th_weight': 0, 'product_uos': False,
            'tax_id': [[6, False, []]],
            'name': product.name})]

        self.assertEqual(len(so.order_line), 0)
        a = None
        with self.assertRaises(osv.except_osv):
            a = self.so_pool.write(
                self.cr, self.user_1.id, so.id, {'order_line': sol})

        so = self.so_pool.browse(
            self.cr, self.user_1.id, so.id)

        self.assertEqual(len(so.order_line), 0)

        pol = [(0, False, {
            'product_id': product.id,
            'name': product.name,
            'date_planned': '10/20/2013',
            'product_qty': 1,
            'price_unit': 885,
            'product_uom': 1,
            'tax_id': [[6, False, []]]
        })]
        self.po_pool.write(
            self.cr, self.user_2.id, po.id, {'order_line': pol})

        so = self.so_pool.browse(
            self.cr, self.user_1.id, so.id)

        self.assertEqual(len(so.order_line), 1)

        po = self.po_pool.browse(
            self.cr, self.user_2.id, po.id)
        self.assertEqual(
            so.order_line[0].product_id, po.order_line[0].product_id)
        self.assertEqual(
            so.order_line[0].product_uom_qty, po.order_line[0].product_qty)

    # def test_05_so_creates_po_updates_so_line_bidirectional(self):
    #     product = self.m.get_object(
    #         self.cr, self.uid, 'product', 'product_product_1')
    #     self.icops_pool.create(
    #         self.cr, self.uid,
    #         {'company_from': self.company_1.id,
    #          'company_to': self.company_2.id,
    #          'obj2obj': 'so2po',
    #          'status': 'draft',
    #          'direction': 'bidirectional'})
    #     sale_id = self.so_pool.create(self.cr, self.user_1.id, {
    #         'partner_id': self.company_2.partner_id.id,
    #         'partner_invoice_id': self.company_2.partner_id.id,
    #         'partner_shipping_id': self.company_2.partner_id.id,
    #         'shop_id': self.shop_1.id,
    #         'date_order': datetime.now().strftime('%Y-%m-%d'),
    #         'pricelist_id': self.user_1.property_product_pricelist.id
    #     })
    #     so = self.so_pool.browse(self.cr, self.user_1.id, sale_id)

    #     po = so.purchase_id
    #     self.assertEqual(so.order_line, po.order_line)
    #     sol = [(0, False, {
    #         'product_uos_qty': 1, 'product_id': product.id,
    #         'product_uom': 1, 'price_unit': 885,
    #         'product_uom_qty': 1, 'th_weight': 0, 'product_uos': False,
    #         'tax_id': [[6, False, []]],
    #         'name': product.name})]
    #     self.so_pool.write(
    #         self.cr, self.user_1.id, so.id, {'order_line': sol})
    #     so = self.so_pool.browse(
    #         self.cr, self.user_1.id, so.id)

    #     self.assertEqual(len(so.order_line), 1)

    #     po = self.po_pool.browse(
    #         self.cr, self.user_2.id, po.id)

    #     self.assertEqual(
    #         so.order_line[0].product_id, po.order_line[0].product_id)
    #     self.assertEqual(
    #         so.order_line[0].product_uom_qty, po.order_line[0].product_qty)

    #     self.assertEqual(po.order_line[0].product_qty, 1)
    #     pol = [(1, po.order_line[0].id, {
    #         'product_qty': 2,
    #     })]
    #     self.po_pool.write(
    #         self.cr, self.user_2.id, po.id, {'order_line': pol})

    #     so = self.so_pool.browse(
    #         self.cr, self.user_1.id, so.id)

    #     self.assertEqual(len(so.order_line), 1)

    #     po = self.po_pool.browse(
    #         self.cr, self.user_2.id, po.id)
    #     self.assertEqual(
    #         so.order_line[0].product_id, po.order_line[0].product_id)
    #     self.assertEqual(
    #         so.order_line[0].product_uom_qty, po.order_line[0].product_qty)
    #     self.assertEqual(so.order_line[0].product_uom_qty, 2)

    # def test_06_delete_so(self):

    #     sale_id = self.so_pool.create(self.cr, self.user_1.id, {
    #         'partner_id': self.company_2.partner_id.id,
    #         'partner_invoice_id': self.company_2.partner_id.id,
    #         'partner_shipping_id': self.company_2.partner_id.id,
    #         'shop_id': self.shop_1.id,
    #         'date_order': datetime.now().strftime('%Y-%m-%d'),
    #         'pricelist_id': self.user_1.property_product_pricelist.id
    #     })
    #     result = self.so_pool.unlink(self.cr, self.user_1.id, sale_id)
    #     self.assertTrue(result)

    def test_07_so_deletes_po(self):

        self.icops_pool.create(
            self.cr, self.uid,
            {'company_from': self.company_1.id,
             'company_to': self.company_2.id,
             'obj2obj': 'so2po',
             'status': 'draft',
             'direction': 'regular'})

        self.icops_pool.create(
            self.cr, self.uid,
            {'company_from': self.company_1.id,
             'company_to': self.company_2.id,
             'obj2obj': 'so2po',
             'status': 'unlink',
             'direction': 'regular'})

        sale_id = self.so_pool.create(self.cr, self.user_1.id, {
            'partner_id': self.company_2.partner_id.id,
            'partner_invoice_id': self.company_2.partner_id.id,
            'partner_shipping_id': self.company_2.partner_id.id,
            'shop_id': self.shop_1.id,
            'date_order': datetime.now().strftime('%Y-%m-%d'),
            'pricelist_id': self.user_1.property_product_pricelist.id
        })
        so = self.so_pool.browse(self.cr, self.user_1.id, sale_id)

        po_ids = self.po_pool.search(
            self.cr, self.uid, [('sale_id', '=', so.id)])

        self.assertEqual(len(po_ids), 1)

        self.so_pool.unlink(
            self.cr, self.user_1.id, so.id)

        po_ids = self.po_pool.search(
            self.cr, self.user_2.id, [('sale_id', '=', so.id)])
        self.assertEqual(len(po_ids), 0)

    def test_07_so_deletes_po_inverse(self):

        self.icops_pool.create(
            self.cr, self.uid,
            {'company_from': self.company_1.id,
             'company_to': self.company_2.id,
             'obj2obj': 'so2po',
             'status': 'draft',
             'direction': 'regular'})

        self.icops_pool.create(
            self.cr, self.uid,
            {'company_from': self.company_1.id,
             'company_to': self.company_2.id,
             'obj2obj': 'so2po',
             'status': 'unlink',
             'direction': 'inverse'})

        sale_id = self.so_pool.create(self.cr, self.user_1.id, {
            'partner_id': self.company_2.partner_id.id,
            'partner_invoice_id': self.company_2.partner_id.id,
            'partner_shipping_id': self.company_2.partner_id.id,
            'shop_id': self.shop_1.id,
            'date_order': datetime.now().strftime('%Y-%m-%d'),
            'pricelist_id': self.user_1.property_product_pricelist.id
        })
        so = self.so_pool.browse(self.cr, self.user_1.id, sale_id)

        po_ids = self.po_pool.search(
            self.cr, self.user_2.id, [('sale_id', '=', so.id)])

        self.assertEqual(len(po_ids), 1)

        so_ids = self.so_pool.search(
            self.cr, self.user_1.id, [('purchase_id', '=', so.purchase_id.id)])

        self.assertEqual(len(po_ids), 1)

        # with self.assertRaises(osv.except_osv):
        #     self.so_pool.unlink(
        #         self.cr, self.user_1.id, so.id)

        # so_ids = self.so_pool.search(
        #     self.cr, self.user_1.id, [('purchase_id', '=', so.purchase_id.id)])

        # self.assertEqual(len(po_ids), 1)

        self.po_pool.unlink(
            self.cr, self.user_2.id, so.purchase_id.id)

        so_ids = self.so_pool.search(
            self.cr, self.user_1.id, [('purchase_id', '=', so.purchase_id.id)])

        self.assertEqual(len(so_ids), 0)

    # def test_08_so_deletes_po_bidirectional_1(self):

    #     self.icops_pool.create(
    #         self.cr, self.uid,
    #         {'company_from': self.company_1.id,
    #          'company_to': self.company_2.id,
    #          'obj2obj': 'so2po',
    #          'status': 'draft',
    #          'direction': 'regular'})

    #     self.icops_pool.create(
    #         self.cr, self.uid,
    #         {'company_from': self.company_1.id,
    #          'company_to': self.company_2.id,
    #          'obj2obj': 'so2po',
    #          'status': 'unlink',
    #          'direction': 'bidirectional'})

    #     sale_id = self.so_pool.create(self.cr, self.user_1.id, {
    #         'partner_id': self.company_2.partner_id.id,
    #         'partner_invoice_id': self.company_2.partner_id.id,
    #         'partner_shipping_id': self.company_2.partner_id.id,
    #         'shop_id': self.shop_1.id,
    #         'date_order': datetime.now().strftime('%Y-%m-%d'),
    #         'pricelist_id': self.user_1.property_product_pricelist.id
    #     })
    #     so = self.so_pool.browse(self.cr, self.user_1.id, sale_id)

    #     po_ids = self.po_pool.search(
    #         self.cr, self.uid, [('sale_id', '=', so.id)])

    #     self.assertEqual(len(po_ids), 1)

    #     self.so_pool.unlink(
    #         self.cr, self.user_1.id, so.id)

    #     po_ids = self.po_pool.search(
    #         self.cr, self.user_2.id, [('sale_id', '=', so.id)])
    #     self.assertEqual(len(po_ids), 0)

    # def test_09_so_deletes_po_bidirectional_2(self):

    #     self.icops_pool.create(
    #         self.cr, self.uid,
    #         {'company_from': self.company_1.id,
    #          'company_to': self.company_2.id,
    #          'obj2obj': 'so2po',
    #          'status': 'draft',
    #          'direction': 'regular'})

    #     self.icops_pool.create(
    #         self.cr, self.uid,
    #         {'company_from': self.company_1.id,
    #          'company_to': self.company_2.id,
    #          'obj2obj': 'so2po',
    #          'status': 'unlink',
    #          'direction': 'bidirectional'})

    #     sale_id = self.so_pool.create(self.cr, self.user_1.id, {
    #         'partner_id': self.company_2.partner_id.id,
    #         'partner_invoice_id': self.company_2.partner_id.id,
    #         'partner_shipping_id': self.company_2.partner_id.id,
    #         'shop_id': self.shop_1.id,
    #         'date_order': datetime.now().strftime('%Y-%m-%d'),
    #         'pricelist_id': self.user_1.property_product_pricelist.id
    #     })
    #     so = self.so_pool.browse(self.cr, self.user_1.id, sale_id)

    #     po_ids = self.po_pool.search(
    #         self.cr, self.uid, [('sale_id', '=', so.id)])

    #     self.assertEqual(len(po_ids), 1)

    #     self.po_pool.unlink(
    #         self.cr, self.user_2.id, so.purchase_id.id)

    #     so_ids = self.so_pool.search(
    #         self.cr, self.user_1.id, [('purchase_id', '=', so.purchase_id.id)])
    #     self.assertEqual(len(so_ids), 0)

    # def test_100_so_creates_po_double_setup(self):
    #     product = self.m.get_object(
    #         self.cr, self.uid, 'product', 'product_product_1')
    #     self.icops_pool.create(
    #         self.cr, self.uid,
    #         {'company_from': self.company_1.id,
    #          'company_to': self.company_2.id,
    #          'obj2obj': 'so2po',
    #          'status': 'draft',
    #          'direction': 'bidirectional'})

    #     self.icops_pool.create(
    #         self.cr, self.uid,
    #         {'company_from': self.company_2.id,
    #          'company_to': self.company_1.id,
    #          'obj2obj': 'po2so',
    #          'status': 'draft',
    #          'direction': 'bidirectional'})

    #     sale_id = self.so_pool.create(self.cr, self.user_1.id, {
    #         'partner_id': self.company_2.partner_id.id,
    #         'partner_invoice_id': self.company_2.partner_id.id,
    #         'partner_shipping_id': self.company_2.partner_id.id,
    #         'shop_id': self.shop_1.id,
    #         'date_order': datetime.now().strftime('%Y-%m-%d'),
    #         'pricelist_id': self.user_1.property_product_pricelist.id
    #     })
    #     so = self.so_pool.browse(self.cr, self.user_1.id, sale_id)

    #     po = so.purchase_id
    #     self.assertEqual(so.order_line, po.order_line)
    #     sol = [(0, False, {
    #         'product_uos_qty': 1, 'product_id': product.id,
    #         'product_uom': 1, 'price_unit': 885,
    #         'product_uom_qty': 1, 'th_weight': 0, 'product_uos': False,
    #         'tax_id': [[6, False, []]],
    #         'name': product.name})]
    #     self.so_pool.write(
    #         self.cr, self.user_1.id, so.id, {'order_line': sol})
    #     so = self.so_pool.browse(
    #         self.cr, self.user_1.id, so.id)

    #     self.assertEqual(len(so.order_line), 1)

    #     po = self.po_pool.browse(
    #         self.cr, self.user_2.id, po.id)
    #     self.assertEqual(
    #         so.order_line[0].product_id, po.order_line[0].product_id)
    #     self.assertEqual(
    #         so.order_line[0].product_uom_qty, po.order_line[0].product_qty)

    # def test_101_so_creates_po_double_setup_inverse(self):
    #     product = self.m.get_object(
    #         self.cr, self.uid, 'product', 'product_product_1')
    #     self.icops_pool.create(
    #         self.cr, self.uid,
    #         {'company_from': self.company_1.id,
    #          'company_to': self.company_2.id,
    #          'obj2obj': 'so2po',
    #          'status': 'draft',
    #          'direction': 'inverse'})

    #     self.icops_pool.create(
    #         self.cr, self.uid,
    #         {'company_from': self.company_2.id,
    #          'company_to': self.company_1.id,
    #          'obj2obj': 'po2so',
    #          'status': 'draft',
    #          'direction': 'inverse'})

    #     sale_id = self.so_pool.create(self.cr, self.user_1.id, {
    #         'partner_id': self.company_2.partner_id.id,
    #         'partner_invoice_id': self.company_2.partner_id.id,
    #         'partner_shipping_id': self.company_2.partner_id.id,
    #         'shop_id': self.shop_1.id,
    #         'date_order': datetime.now().strftime('%Y-%m-%d'),
    #         'pricelist_id': self.user_1.property_product_pricelist.id
    #     })
    #     so = self.so_pool.browse(self.cr, self.user_1.id, sale_id)

    #     po = so.purchase_id
    #     self.assertEqual(so.order_line, po.order_line)
    #     sol = [(0, False, {
    #         'product_uos_qty': 1, 'product_id': product.id,
    #         'product_uom': 1, 'price_unit': 885,
    #         'product_uom_qty': 1, 'th_weight': 0, 'product_uos': False,
    #         'tax_id': [[6, False, []]],
    #         'name': product.name})]
    #     self.so_pool.write(
    #         self.cr, self.user_1.id, so.id, {'order_line': sol})
    #     so = self.so_pool.browse(
    #         self.cr, self.user_1.id, so.id)

    #     self.assertEqual(len(so.order_line), 1)

    #     po = self.po_pool.browse(
    #         self.cr, self.user_2.id, po.id)
    #     self.assertEqual(
    #         so.order_line[0].product_id, po.order_line[0].product_id)
    #     self.assertEqual(
    #         so.order_line[0].product_uom_qty, po.order_line[0].product_qty)
